/* font test -- based on extext */

/* Specifically, this tests alpha fonts blending into backgrounds */


#include <time.h>
#include <math.h>
#include <string.h>

#include "allegro.h"
#include "alleggl.h"


#ifndef M_PI
#define M_PI 3.1415
#endif

#define NUM_STRINGS 3
#define STRING "Hello World!"

#define STRING0 ("AllegroGL  v" AGL_VERSION_STR)
#define STRING1 "Text Drawing"
#define STRING2 "And Font Loading"
#define STRING3 "Example Program"
#define STRING4 "See Our Webpage At:"


#define URL "http://allegrogl.sourceforge.net/"

volatile int chrono = 0;
volatile double angle = 0.0;

void the_timer(void) {
	chrono++;
	angle += 1.0 / 400.0;

	if (angle > M_PI)
		angle -= 2 * M_PI;

} END_OF_FUNCTION(the_timer);



int main() {

	FONT *demofont;
	DATAFILE *dat;
	
	allegro_init();
	install_allegro_gl();
	install_timer();
	
	LOCK_FUNCTION(the_timer);
	LOCK_VARIABLE(chrono);
	
	install_int(the_timer, 2);

	allegro_gl_clear_settings();
	allegro_gl_set(AGL_COLOR_DEPTH, 32);
	allegro_gl_set(AGL_WINDOWED, TRUE);
	allegro_gl_set(AGL_DOUBLEBUFFER, 1);
	allegro_gl_set(AGL_SUGGEST, AGL_DOUBLEBUFFER | AGL_COLOR_DEPTH
	             | AGL_WINDOWED);

	set_color_depth(32);
	if (set_gfx_mode(GFX_OPENGL, 640, 480, 0, 0)) {
		allegro_message("Error initializing OpenGL!\n");
		return -1;
	}

	install_keyboard();

	/* Load font palette first, then font itself */
	dat = load_datafile_object ("demofont.dat", "PAL");
	if (!dat) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error loading demofont.dat#PAL\n");
		return -1;
	}
	set_palette ((RGB *)dat->dat);
	unload_datafile_object (dat);

	dat = load_datafile_object ("demofont.dat", "FONT");
	if (!dat) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error loading demofont.dat#FONT\n");
		return -1;
	}
	allegro_gl_set_texture_format (GL_ALPHA8);
	demofont = allegro_gl_convert_allegro_font((FONT*)dat->dat,
	                                            AGL_FONT_TYPE_TEXTURED, 16.0);
	unload_datafile_object(dat);

	if (!demofont) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Unable to convert font!\n");
		return -1;
	}


	/* Setup OpenGL like we want */
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE);

	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT, GL_FILL);

	glViewport(0, 0, SCREEN_W, SCREEN_H);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, -1, 1);

	glMatrixMode(GL_MODELVIEW);

	do {
		int x, y;
		glLoadIdentity();
		
		/* Draw the background */
		for (y = 0; y < 4; y++)
			for (x = 0; x < 4; x++) {
				glColor3f ((y&1), (x&1), ((y&2)^(x&2))/2);
				glBegin (GL_QUADS);
					glVertex2f (x*0.25, y*0.25);
					glVertex2f (x*0.25, (y+1)*0.25);
					glVertex2f ((x+1)*0.25, (y+1)*0.25);
					glVertex2f ((x+1)*0.25, y*0.25);
				glEnd();
			}

    	/* Draw the text on top */
    	glScalef (0.1, 0.1, 0.1);
    	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    	allegro_gl_printf(demofont, 1, 8, 0, makeacol32(255,255,255,255),
		                 "Hello World!");
    	glBlendFunc(GL_ONE, GL_ZERO);
	
		allegro_gl_flip();

	} while (!keypressed());
	
	allegro_gl_destroy_font(demofont);

	return 0;
}
END_OF_MAIN();

